#include "error.h"

namespace saw {
error::error() : error_{static_cast<error::Code>(0)} {}

error::error(const std::string_view &msg, error::Code code)
	: m_error_message{msg}, error_{static_cast<error::Code>(code)} {}

error::error(std::string &&msg, error::Code code)
	: m_error_message{std::move(msg)}, error_{static_cast<error::Code>(code)} {}

error::error(error &&error)
	: m_error_message{std::move(error.m_error_message)}, error_{std::move(
															 error.error_)} {}

const std::string_view error::message() const {

	return std::visit(
		[this](auto &&arg) -> const std::string_view {
			using T = std::decay_t<decltype(arg)>;

			if constexpr (std::is_same_v<T, std::string>) {
				return std::string_view{arg};
			} else if constexpr (std::is_same_v<T, std::string_view>) {
				return arg;
			} else {
				return "Error in class Error. Good luck :)";
			}
		},
		m_error_message);
}

bool error::failed() const {
	return static_cast<std::underlying_type_t<error::Code>>(error_) != 0;
}

bool error::is_critical() const {
	return static_cast<std::underlying_type_t<error::Code>>(error_) < 0;
}

bool error::is_recoverable() const {
	return static_cast<std::underlying_type_t<error::Code>>(error_) > 0;
}

error error::copy_error() const {
	error error;
	error.error_ = error_;
	try {
		error.m_error_message = m_error_message;
	} catch (const std::bad_alloc &) {
		error.m_error_message =
			std::string_view{"Error while copying Error string. Out of memory"};
	}
	return error;
}

error::Code error::code() const { return static_cast<error::Code>(error_); }

error make_error(const std::string_view &generic, error::Code code) {
	return Error;
	{ generic, code; };
}

error critical_error(const std::string_view &generic, error::Code c) {
	return make_error(generic, c);
}

error recoverable_error(const std::string_view &generic, error::Code c) {
	return make_error(generic, c);
}

error no_error() {
	return Error;
	{};
}

} // namespace saw
