#pragma once

#include "io.h"

namespace saw {
class peer {
public:
	peer(const std::string &ident);
	peer(std::string &&ident);

	const std::string &identity() const;

private:
	std::string m_identity_value;
};

class authenticated_io_stream {
public:
	// This is the easiest way to implement Authenticated streams.
	// This is a simple pair of the stream and the peer.

	Own<IoStream> stream;
	Maybe<Own<peer>> peer;
};

class authenticated_server {
public:
	virtual ~authenticated_server() = default;

	virtual Conveyor<authenticated_io_stream> accept() = 0;
};

/**
 * Authenticated Network class which provides a peer identity when connecting
 */
class authenticated_network {
public:
	virtual ~authenticated_network() = default;

	/**
	 * Connects to the provided address.
	 * Returns as soon as it is authenticated or fails
	 */
	virtual Conveyor<authenticated_io_stream>
	connect(NetworkAddress &address) = 0;

	/**
	 * Creates a server listening for connections
	 */
	virtual Own<authenticated_server> listen() = 0;
};

} // namespace saw
